@@ -91,6 +91,26 @@ test_resolution_real (const char *base_uri,
 static void
 test_resolution (void)
 {
+	const char * const *schemes;
+	gboolean http_supported = FALSE;
+	GVfs *vfs;
+	guint i;
+
+	vfs = g_vfs_get_default ();
+	if (vfs == NULL)
+		g_error ("gvfs with http support is required to test link resolution");
+	schemes = g_vfs_get_supported_uri_schemes (vfs);
+	if (schemes == NULL)
+		g_error ("gvfs with http support is required to test link resolution");
+	for (i = 0; schemes[i] != NULL; i++) {
+		if (g_str_equal (schemes[i], "http")) {
+			http_supported = TRUE;
+			break;
+		}
+	}
+	if (http_supported == FALSE)
+		g_error ("gvfs is installed but does not support http");
+
 	/* http://bugzilla.gnome.org/show_bug.cgi?id=555417 */
 	g_assert_cmpstr (test_resolution_real ("http://www.yle.fi/player/player.jsp", "288629.asx?s=1000"), ==, "http://www.yle.fi/player/288629.asx?s=1000");
 	g_assert_cmpstr (test_resolution_real ("http://www.yle.fi/player/player.jsp?actionpage=3&id=288629&locale", "288629.asx?s=1000"), ==, "http://www.yle.fi/player/288629.asx?s=1000");
@@ -114,6 +134,8 @@ test_duration (void)
 	g_assert_cmpint (totem_pl_parser_parse_duration ("01:00:01.01", verbose), ==, 3601);
 	g_assert_cmpint (totem_pl_parser_parse_duration ("01:00.01", verbose), ==, 60);
 	g_assert_cmpint (totem_pl_parser_parse_duration ("24.59", verbose), ==, 1499);
+	g_assert_cmpint (totem_pl_parser_parse_duration ("02m25s", verbose), ==, 145);
+	g_assert_cmpint (totem_pl_parser_parse_duration ("2m25s", verbose), ==, 145);
 }
 
 static void
@@ -189,6 +211,17 @@ test_data_get_data (const char *uri, guint *len)
 	return buffer;
 }
 
+static void
+test_videosite (void)
+{
+#ifdef HAVE_QUVI
+	char *uri = "http://www.youtube.com/watch?v=oMLCrzy9TEs";
+
+	g_test_message ("Testing data parsing \"%s\"...", uri);
+	g_assert (totem_pl_parser_can_parse_from_uri (uri, TRUE));
+#endif
+}
+
 static void
 test_parsability (void)
 {
@@ -206,6 +239,7 @@ test_parsability (void)
 		{ "http://live.hujjat.org:7860/main", FALSE, TRUE },
 		{ "http://www.comedycentral.com/sitewide/media_player/videoswitcher.jhtml?showid=934&category=/shows/the_daily_show/videos/headlines&sec=videoId%3D36032%3BvideoFeatureId%3D%3BpoppedFrom%3D_shows_the_daily_show_index.jhtml%3BisIE%3Dfalse%3BisPC%3Dtrue%3Bpagename%3Dmedia_player%3Bzyg%3D%27%2Bif_nt_zyg%2B%27%3Bspan%3D%27%2Bif_nt_span%2B%27%3Bdemo%3D%27%2Bif_nt_demo%2B%27%3Bbps%3D%27%2Bif_nt_bandwidth%2B%27%3Bgateway%3Dshows%3Bsection_1%3Dthe_daily_show%3Bsection_2%3Dvideos%3Bsection_3%3Dheadlines%3Bzyg%3D%27%2Bif_nt_zyg%2B%27%3Bspan%3D%27%2Bif_nt_span%2B%27%3Bdemo%3D%27%2Bif_nt_demo%2B%27%3Bera%3D%27%2Bif_nt_era%2B%27%3Bbps%3D%27%2Bif_nt_bandwidth%2B%27%3Bfla%3D%27%2Bif_nt_Flash%2B%27&itemid=36032&clip=com/dailyshow/headlines/10156_headline.wmv&mswmext=.asx", TRUE, TRUE },
 		{ TEST_SRCDIR "HackerMedley", TRUE, FALSE }, /* From https://bugzilla.redhat.com/show_bug.cgi?id=582850 and http://feeds.feedburner.com/HackerMedley */
+		{ "http://faif.us/feeds/cast-mp3", TRUE, TRUE },
 		{ NULL,  FALSE, FALSE }
 	};
 
@@ -651,6 +685,18 @@ test_parsing_not_asx_playlist (void)
 	g_free (uri);
 }
 
+static void
+test_parsing_wma_asf (void)
+{
+	char *uri;
+	g_test_bug ("639958");
+	/* File from https://bugzilla.gnome.org/show_bug.cgi?id=639958#c5 */
+	uri = get_relative_uri (TEST_SRCDIR "WMA9.1_98_quality_48khz_vbr_s.wma");
+	g_assert (simple_parser_test (uri) == TOTEM_PL_PARSER_RESULT_SUCCESS);
+	g_assert_cmpstr (parser_test_get_entry_field (uri, TOTEM_PL_PARSER_FIELD_URI), ==, "mmsh://195.134.224.231/wma/WMA9.1_98_quality_48khz_vbr_s.wma?MSWMExt=.asf");
+	g_free (uri);
+}
+
 static void
 test_parsing_not_really_php (void)
 {
@@ -673,6 +719,73 @@ test_parsing_not_really_php_but_html_instead (void)
 	g_free (uri);
 }
 
+
+typedef struct {
+	int count;
+	GMainLoop *mainloop;
+	char *uri;
+} AsyncParseData;
+
+static void
+parse_async_ready (GObject *pl, GAsyncResult *result, gpointer userdata)
+{
+	AsyncParseData *data = userdata;
+	TotemPlParserResult retval;
+
+	retval = totem_pl_parser_parse_finish (TOTEM_PL_PARSER (pl), result, NULL);
+	g_test_message ("Got retval %d for uri '%s'", retval, data->uri);
+	g_test_message ("Parsed entry count is %d for uri '%s'", data->count, data->uri);
+
+	g_assert (data->count > 0);
+
+	g_main_loop_quit (data->mainloop);
+	g_object_unref (pl);
+}
+
+static gboolean
+block_main_loop_idle (gpointer data)
+{
+	/* one second should be long enough to parse a local file */
+	g_usleep (G_USEC_PER_SEC);
+	return FALSE;
+}
+
+static void
+test_async_parsing_signal_order (void)
+{
+	AsyncParseData data;
+	TotemPlParser *pl = totem_pl_parser_new ();
+
+	g_test_bug ("631727");
+	/* doesn't really matter what test file we use here, we just want something
+	 * with at least one valid entry in it.
+	 */
+	data.uri = get_relative_uri (TEST_SRCDIR "separator.m3u");
+	data.mainloop = g_main_loop_new (NULL, FALSE);
+	data.count = 0;
+
+	g_object_set (pl, "recurse", FALSE,
+			  "debug", option_debug,
+			  "force", option_force,
+			  "disable-unsafe", option_disable_unsafe,
+			  NULL);
+	g_signal_connect (G_OBJECT (pl), "entry-parsed",
+			  G_CALLBACK (entry_parsed_num_cb), &data.count);
+
+	/* we need the mainloop to be busy while the signal emission idle sources are
+	 * added, so we just block it for a while.
+	 */
+	g_idle_add_full (G_PRIORITY_HIGH, block_main_loop_idle, NULL, NULL);
+	totem_pl_parser_parse_async (pl, data.uri, FALSE, NULL, parse_async_ready, &data);
+	g_main_loop_run (data.mainloop);
+
+	/* The number of entries in separator.m3u */
+	g_assert(data.count == 1);
+
+	g_free (data.uri);
+	g_main_loop_unref (data.mainloop);
+}
+
 #define MAX_DESCRIPTION_LEN 128
 #define DATE_BUFSIZE 512
 #define PRINT_DATE_FORMAT "%Y-%m-%dT%H:%M:%SZ"
@@ -827,6 +940,7 @@ main (int argc, char *argv[])
 		g_test_add_func ("/parser/relative", test_relative);
 		g_test_add_func ("/parser/resolution", test_resolution);
 		g_test_add_func ("/parser/parsability", test_parsability);
+		g_test_add_func ("/parser/videosite", test_videosite);
 		g_test_add_func ("/parser/parsing/hadess", test_parsing_hadess);
 		g_test_add_func ("/parser/parsing/nonexistent_files", test_parsing_nonexistent_files);
 		g_test_add_func ("/parser/parsing/broken_asx", test_parsing_broken_asx);
@@ -852,6 +966,8 @@ main (int argc, char *argv[])
 		g_test_add_func ("/parser/parsing/m3u_leading_tabs", test_m3u_leading_tabs);
 		g_test_add_func ("/parser/parsing/empty-asx.asx", test_empty_asx);
 		g_test_add_func ("/parser/parsing/dir_recurse", test_directory_recurse);
+		g_test_add_func ("/parser/parsing/async_signal_order", test_async_parsing_signal_order);
+		g_test_add_func ("/parser/parsing/wma_asf", test_parsing_wma_asf);
 
 		return g_test_run ();
 	}
