@@ -221,6 +221,8 @@ totem_pl_parser_load_directory (GFile *file, GList **list, gboolean *unhandled)
 	while ((info = g_file_enumerator_next_file (e, NULL, NULL)) != NULL)
 		*list = g_list_prepend (*list, info);
 
+	g_file_enumerator_close (e, NULL, NULL);
+	g_object_unref (e);
 	return TRUE;
 }
 
@@ -271,7 +273,9 @@ totem_pl_parser_add_directory (TotemPlParser *parser,
 		item = g_file_get_child (file, g_file_info_get_name (info));
 
 		ret = totem_pl_parser_parse_internal (parser, item, NULL, parse_data);
-		if (ret != TOTEM_PL_PARSER_RESULT_SUCCESS && ret != TOTEM_PL_PARSER_RESULT_IGNORED) {
+		if (ret != TOTEM_PL_PARSER_RESULT_SUCCESS &&
+		    ret != TOTEM_PL_PARSER_RESULT_IGNORED &&
+		    ret != TOTEM_PL_PARSER_RESULT_ERROR) {
 			char *item_uri;
 
 			item_uri = g_file_get_uri (item);
