@@ -153,6 +153,7 @@
 #include "totem-pl-parser-lines.h"
 #include "totem-pl-parser-misc.h"
 #include "totem-pl-parser-private.h"
+#include "totem-pl-parser-videosite.h"
 
 #define READ_CHUNK_SIZE 8192
 #define RECURSE_LEVEL_MAX 4
@@ -230,6 +231,7 @@ static PlaylistTypes dual_types[] = {
 	PLAYLIST_TYPE2 ("audio/x-ms-asx", totem_pl_parser_add_asx, totem_pl_parser_is_asx),
 	PLAYLIST_TYPE2 ("video/x-ms-asf", totem_pl_parser_add_asf, totem_pl_parser_is_asf),
 	PLAYLIST_TYPE2 ("video/x-ms-wmv", totem_pl_parser_add_asf, totem_pl_parser_is_asf),
+	PLAYLIST_TYPE2 ("audio/x-ms-wma", totem_pl_parser_add_asf, totem_pl_parser_is_asf),
 	PLAYLIST_TYPE2 ("video/quicktime", totem_pl_parser_add_quicktime, totem_pl_parser_is_quicktime),
 	PLAYLIST_TYPE2 ("video/mp4", totem_pl_parser_add_quicktime, totem_pl_parser_is_quicktime),
 	PLAYLIST_TYPE2 ("application/x-quicktime-media-link", totem_pl_parser_add_quicktime, totem_pl_parser_is_quicktime),
@@ -280,7 +282,6 @@ enum {
 
 static int totem_pl_parser_table_signals[LAST_SIGNAL];
 static GParamSpecPool *totem_pl_parser_pspec_pool = NULL;
-static gboolean i18n_done = FALSE;
 
 static void totem_pl_parser_class_init (TotemPlParserClass *klass);
 static void totem_pl_parser_base_class_finalize	(TotemPlParserClass *klass);
@@ -645,15 +646,19 @@ totem_pl_parser_error_quark (void)
 	return quark;
 }
 
+static gpointer
+totem_pl_parser_real_init_i18n (gpointer data)
+{
+	bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
+	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+	return NULL;
+}
+
 static void
 totem_pl_parser_init_i18n (void)
 {
-	if (i18n_done == FALSE) {
-		/* set up translation catalog */
-		bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
-		bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
-		i18n_done = TRUE;
-	}
+	static GOnce my_once = G_ONCE_INIT;
+	g_once (&my_once, totem_pl_parser_real_init_i18n, NULL);
 }
 
 /**
@@ -714,7 +719,7 @@ static char *
 my_g_file_info_get_mime_type_with_data (GFile *file, gpointer *data, TotemPlParser *parser)
 {
 	char *buffer;
-	gssize bytes_read;
+	gsize bytes_read;
 	GFileInputStream *stream;
 	GError *error = NULL;
 
@@ -751,13 +756,13 @@ my_g_file_info_get_mime_type_with_data (GFile *file, gpointer *data, TotemPlPars
 
 	/* Read the whole thing, up to MIME_READ_CHUNK_SIZE */
 	buffer = g_malloc (MIME_READ_CHUNK_SIZE);
-	bytes_read = g_input_stream_read (G_INPUT_STREAM (stream), buffer, MIME_READ_CHUNK_SIZE, NULL, &error);
-	g_object_unref (G_INPUT_STREAM (stream));
-	if (bytes_read == -1) {
+	if (g_input_stream_read_all (G_INPUT_STREAM (stream), buffer, MIME_READ_CHUNK_SIZE, &bytes_read, NULL, &error) == FALSE) {
+		g_object_unref (stream);
 		DEBUG(file, g_print ("Couldn't read data from '%s'\n", uri));
 		g_free (buffer);
 		return NULL;
 	}
+	g_object_unref (G_INPUT_STREAM (stream));
 
 	/* Empty file */
 	if (bytes_read == 0) {
@@ -1785,6 +1790,20 @@ totem_pl_parser_parse_internal (TotemPlParser *parser,
 	if (!parse_data->recurse && parse_data->recurse_level > 0)
 		return TOTEM_PL_PARSER_RESULT_UNHANDLED;
 
+#ifdef HAVE_QUVI
+	/* Should we try to parse it with quvi? */
+	if (g_file_has_uri_scheme (file, "http")) {
+		char *url;
+		url = g_file_get_uri (file);
+		if (url != NULL && totem_pl_parser_is_videosite (url, parser->priv->debug) != FALSE) {
+			ret = totem_pl_parser_add_videosite (parser, file, base_file, parse_data, NULL);
+			if (ret == TOTEM_PL_PARSER_RESULT_SUCCESS)
+				return ret;
+		}
+		g_free (url);
+	}
+#endif /* HAVE_QUVI */
+
 	/* In force mode we want to get the data */
 	if (parse_data->force != FALSE) {
 		mimetype = my_g_file_info_get_mime_type_with_data (file, &data, parser);
@@ -2250,6 +2269,11 @@ totem_pl_parser_parse_duration (const char *duration, gboolean debug)
 		D(g_print ("Used broken float format (00.00)\n"));
 		return minutes * 60 + seconds;
 	}
+	/* YouTube format */
+	if (sscanf (duration, "%dm%ds", &minutes, &seconds) == 2) {
+		D(g_print ("Used YouTube format\n"));
+		return minutes * 60 + seconds;
+	}
 	/* PLS files format */
 	if (sscanf (duration, "%d", &seconds) == 1) {
 		D(g_print ("Used PLS format\n"));
@@ -2439,6 +2463,23 @@ totem_pl_parser_can_parse_from_filename (const char *filename, gboolean debug)
 	return retval;
 }
 
+/**
+ * totem_pl_parser_can_parse_from_uri:
+ * @uri: the remote URI to check for parsability
+ * @debug: %TRUE if debug statements should be printed
+ *
+ * Checks if the remote URI can be parsed. Note that this does
+ * not actually try to open the remote URI, or deduce its mime-type
+ * from filename, as this would bring too many false positives.
+ *
+ * Return value: %TRUE if @uri could be parsed
+ **/
+gboolean
+totem_pl_parser_can_parse_from_uri (const char *uri, gboolean debug)
+{
+	return totem_pl_parser_is_videosite (uri, debug);
+}
+
 #ifndef TOTEM_PL_PARSER_MINI
 GType
 totem_pl_parser_metadata_get_type (void)
