@@ -26,14 +26,17 @@
 #ifdef XINE_COMPILE
 # include "config.h"
 #endif
-
-#include <unistd.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <stdarg.h>
 #include <ctype.h>
 
+#ifdef _MSC_VER
+#define snprintf sprintf_s
+#define strcasecmp stricmp
+#endif
+
 #define LOG_MODULE "xmlparser"
 #define LOG_VERBOSE
 /*
@@ -53,7 +56,7 @@
 
 #define TOKEN_SIZE  64 * 1024
 #define DATA_SIZE   64 * 1024
-#define MAX_RECURSION 23
+#define MAX_RECURSION 26
 
 /* private global variables */
 xml_parser_t * static_xml_parser;
@@ -839,13 +842,14 @@ char *xml_escape_string (const char *s, xml_escape_quote_t quote_type)
 }
 
 static void xml_parser_dump_node (const xml_node_t *node, int indent) {
+  size_t l;
 
   xml_property_t *p;
   xml_node_t     *n;
 
   printf ("%*s<%s ", indent, "", node->name);
 
-  size_t l = strlen (node->name);
+  l = strlen (node->name);
 
   p = node->props;
   while (p) {
